import time
import socket
import threading
import struct
import json
import websockets
import asyncio
import sched

def discoverRoversUDPtx():
    #sends 0x80 0x00 0x00 to all the rovers a bunch of times to evoke a response
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    s.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
    lp=0
    while(lp<20):
        s.sendto(b'\x80\x00\x00', ('255.255.255.255', 7700)) #broadcast the discover command
        time.sleep(0.1)
        lp+=1    

def discoverRovers(): #looks for rovers on the network
    th = threading.Thread(target=discoverRoversUDPtx)
    #reads the responses from the UDP packets sent back from the rovers
    s=socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.bind(('',7700))
    s.setblocking(0)
    th.start()
    timeout = time.time()+3 #set a few second timer
    rxraw = []
    while(time.time()<timeout):
        try:
            m,addr=s.recvfrom(32)
            rxraw.append(m)
        except:
            pass
    th.join()
    rxfilt = list(dict.fromkeys(rxraw)) #remove duplicate entries
    rxfilt = list(filter((b'\x80\x00\x00').__ne__, rxfilt)) #remove the trigger command that we
    vv = []
    for item in rxfilt:
        name = item[1:9].decode()
        ipb = item[9:13]
        ip = str(ipb[0])+'.'+str(ipb[1])+'.'+str(ipb[2])+'.'+str(ipb[3])
        ent = {"name":name, "ip":ip}
        vv.append(ent)
    return vv

class rov: #class for control of the esp rovers
    def clamp(self, n, minn, maxn): #clamp / constrain value to between min/max
        return max(min(maxn, n), minn)
    def __init__(self, ip:str, name:str):
        self.ip = ip
        self.name = name
        self.lock = False #inuse lock
        self.clawLock=False #claw disabler
    def drive(self, left, right, drivetime): #directions, drive time
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM) # UDP
        lm = struct.pack('b',self.clamp(left,-1,1))
        rm = struct.pack('b',self.clamp(right,-1,1))
        dt = struct.pack('>H',self.clamp(drivetime,0,65535))
        msg = b'\x00'+rm+lm+dt
        sock.sendto(msg, (self.ip, 7700))
    def lamp(self, state): #lamp on (1) or off (0)
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM) # UDP
        st = struct.pack('b',self.clamp(state,0,1))
        msg = b'\x01'+st
        sock.sendto(msg, (self.ip, 7700))
    def servo(self, setting): #servo open/close by presets on device
        if(self.clawLock==False):
            sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM) # UDP
            st = struct.pack('B',self.clamp(setting,0,7))
            msg = b'\x03'+st
            sock.sendto(msg, (self.ip, 7700))

def autoInitRovers(): #initializes the array of rover objects using discovery
    rovers = []
    dis = discoverRovers()
    for r in dis:
        rovers.append(rov(r['ip'],r['name']))
    return rovers

print('searching for rovers...')
rovers = autoInitRovers()

for r in rovers:
    print(r.name,' ',r.ip)

wsClients = set(); #list of all the websocket clients
async def register(websocket):
    wsClients.add(websocket)
async def unregister(websocket):
    wsClients.remove(websocket)

async def rovlockstat():
    print("sending lock status:")
    rl = []
    for i in rovers:
        rl.append({'name': i.name, 'lock': i.lock})
        print('\t',i.name,' ',i.lock)
        resp = {"rovlockstat": rl}
    for x in wsClients:
        await x.send(json.dumps(resp))

async def wserver(websocket, path):
    await register(websocket)
    try:
       async for message in websocket:
            resp = []
            data = json.loads(message)
            
            if data['reqtype'] == 'dash':
                #return with a list of all the names and ip addresses
                print('sending dashboard')
                rl = []
                for i in rovers:
                    rl.append({'name': i.name, 'ip': i.ip})
                resp = {"dash": rl}
                await websocket.send(json.dumps(resp))
                
            if data['reqtype'] == 'rovlockstat':
                await rovlockstat()
            if data['reqtype'] == 'rovlock':
                print('lock ',data['name'])
                for x in rovers:
                    if x.name == data['name']:
                        x.lock = True
                await rovlockstat()
            if data['reqtype'] == 'rovunlock':
                print('unlock',data['name'])
                for x in rovers:
                    if x.name == data['name']:
                        x.lock = False
                await rovlockstat()

            if data['reqtype'] == 'drive':
                for x in rovers:
                    if x.name == data['name']:
                        l = int(data['l'])
                        r = int(data['r'])
                        t = int(data['t'])
                        print('drive ',x.name,' ',l,',',r,',',t)
                        x.drive(l,r,t)
            if data['reqtype'] == 'claw':
                for x in rovers:
                    if x.name == data['name']:
                        v = int(data['v'])
                        print('claw ',x.name,' ',v)
                        x.servo(v)
            if data['reqtype'] == 'led':
                for x in rovers:
                    if x.name == data['name']:
                        v = int(data['v'])
                        print('led ',x.name,' ',v)
                        x.lamp(v)
            if data['reqtype'] == 'refresh':
                print('refresh', data['name'])
                resp = {"refresh": data['name']}
                for x in wsClients:
                    await x.send(json.dumps(resp))
            if data['reqtype'] == 'unlockall':
                for x in rovers:
                    x.lock = False
                await rovlockstat()
            if data['reqtype'] == 'clawLock':
                print('clawLock: ',data['lock'])
                if(data['lock']==1):
                    for x in rovers:
                        x.clawLock = False
                        x.servo(0)
                        x.clawLock = True
                else:
                    for x in rovers:
                        x.clawLock = False
            if data['reqtype'] == 'openClaws':
                    for x in rovers:
                        b = x.clawLock
                        x.clawLock = False
                        x.servo(0)
                        x.clawLock = b
    finally:
        await unregister(websocket)

print("running server")
start_server = websockets.serve(wserver, '192.168.1.122', 1977)
asyncio.get_event_loop().run_until_complete(start_server)
asyncio.get_event_loop().run_forever()





        
