from flask import Flask, request
import time
import socket
import struct


class rov: #class for control of the esp rovers
    def clamp(self, n, minn, maxn): #clamp / constrain value to between min/max
        return max(min(maxn, n), minn)
    def __init__(self, ip:str):
        self.ip = ip
        self.drivetime1 = 500  #for drive time (millis)
        self.drivetime2 = 100  #for drive time (millis) - for turning
    def drive(self, left, right, drivetime): #directions, drive time
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM) # UDP
        lm = struct.pack('b',self.clamp(left,-1,1))
        rm = struct.pack('b',self.clamp(right,-1,1))
        dt = struct.pack('>H',self.clamp(drivetime,0,65535))
        msg = b'\x00'+rm+lm+dt
        sock.sendto(msg, (self.ip, 7700))
    def lamp(self, state): #lamp on (1) or off (0)
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM) # UDP
        st = struct.pack('b',self.clamp(state,0,1))
        msg = b'\x01'+st
        sock.sendto(msg, (self.ip, 7700))
    def servo(self, angle): #servo angle for grabber [0 = full open, 130 = full close]
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM) # UDP
        st = struct.pack('B',self.clamp(angle,0,130))
        msg = b'\x02'+st
        sock.sendto(msg, (self.ip, 7700))

rovers = []
rovers.append(rov('192.168.1.126')) #robot[0]
        
app = Flask(__name__)

@app.route('/esprov/cmd')
def esprov():
    resp = 'err'    
    try:
        rid = int(request.args['id'])
        if(rid < len(rovers)):
            #id is for a valid robot
            resp = 'ok'
            if 'li' in request.args:
                rovers[rid].lamp(int(request.args['li'])) #headlight led lamp
            if 'sv' in request.args:
                rovers[rid].servo(int(request.args['sv'])) #servo position
            if 'dtd' in request.args:
                rovers[rid].drivetime1= int(request.args['dtd']) #drive time (drive)
            if 'dtt' in request.args:
                rovers[rid].drivetime2= int(request.args['dtt']) #drive time (turn)
            if 'lm' and 'rm' in request.args: #command to drive wheels
                lm = int(request.args['lm'])
                rm = int(request.args['rm'])
                if(lm!=rm):
                    print("rover",rid," trn",lm,rm,rovers[rid].drivetime2)
                    rovers[rid].drive(lm,rm,rovers[rid].drivetime2)
                else:
                    print("rover",rid," drv",lm,rm,rovers[rid].drivetime1)
                    rovers[rid].drive(lm,rm,rovers[rid].drivetime1)

    except:
        print("generic error message lol")
        pass
    return resp

if __name__ == '__main__':
   app.run('0.0.0.0',82)
